/*jslint node: true */
'use strict';

/*
 * Copyright 2014 the original author or authors
 * @license MIT, see LICENSE for details
 *
 * @author Artem Abashev
 */

var when = require('when'),
    rest = require('rest'),
    mime = require('rest/interceptor/mime'),
    errorCode = require('rest/interceptor/errorCode'),
    interceptor = require('rest/interceptor');

//------------------------------------------------------------------------
//
//  OAuthInterceptor
//
//------------------------------------------------------------------------

var OAuthInterceptor = interceptor({
  init: function (config) {
    return config;
  },
  request: function (request, config) {
    return config.tokenProvider().then(function(token) {
      if (!request.headers) {
        request.headers = {};
      }

      if (token) {
        request.headers.Authorization = 'Bearer ' + token;
      }

      return request;
    });
  }
});

//------------------------------------------------------------------------
//
//  TouchApiClient
//
//------------------------------------------------------------------------

var TouchApiClient = function(config, tokenProvider) {
  if (!config) {
    throw new Error('Missing configuration');
  }

  if (!tokenProvider || typeof tokenProvider !== 'function') {
    throw new Error('Missing token provider');
  }

  this._config = {
    host: {
      secure: Boolean(config.host.secure),
      domain: config.host.domain.toString()
    },
    path: config.path,
    contentType: 'application/json'
  };

  this._tokenProvider = tokenProvider;
};

TouchApiClient.prototype._getUrl = function(path, params) {
  var url = 'http' +
    (this._config.host.secure ? 's' : '') +
    '://' +
    this._config.host.domain +
    this._config.path +
    path;

    if (params) {
      var query = [];
      for (var p in params) {
        query.push(encodeURIComponent(p) + '=' + encodeURIComponent(params[p]));
      }

      if (query.length > 0) {
        url += '?' + query.join('&');
      }
    }

  return url;
};

TouchApiClient.prototype._request = function(path, method, body, params, unauthorized, contentType) {
  var url = this._getUrl(path);
  var client = rest
    .wrap(errorCode)
    .wrap(mime, { mime: contentType || this._config.contentType });

  if (!unauthorized) {
    client = client.wrap(OAuthInterceptor, {
      tokenProvider: this._tokenProvider
    });
  }

  return client({
    path: url,
    method: method,
    entity: body || {},
    params: params || {}
  }).then(function(response) {
    return response.entity;
  }, function(e) {
    var error = {
      code: 503,
      message: null
    };

    if (e.entity) {
      try
      {
        error = JSON.parse(e.entity);
      }
      catch(e) {
        error.message = e.entity.toString();
      }
    }

    if (e.status) {
      error.code = e.status.code;
    }

    return when.reject(error);
  });
};

module.exports = TouchApiClient;
