/*jslint node: true */
/*global $, Promise, FormData */
'use strict';

/*
 * Copyright 2014 the original author or authors
 * @license MIT, see LICENSE for details
 *
 * @author Artem Abashev
 */

//------------------------------------------------------------------------
//
//  TouchApiClient
//
//------------------------------------------------------------------------

var TouchApiClient = function(config, tokenProvider) {
  if (!config) {
    throw new Error('Missing configuration');
  }

  if (!tokenProvider) {
    throw new Error('Missing token provider');
  }

  this._config = {
    host: {
      secure: Boolean(config.host.secure),
      domain: config.host.domain
    },
    path: config.path,
    contentType: "application/json"
  };

  this._tokenProvider = tokenProvider;
};

TouchApiClient.prototype._getUrl = function(path, params) {
  var url = 'http' +
    (this._config.host.secure ? 's' : '') +
    '://' +
    this._config.host.domain +
    this._config.path +
    path;

  if (params) {
    var query = [];
    for (var p in params) {
      query.push(encodeURIComponent(p) + '=' + encodeURIComponent(params[p]));
    }

    if (query.length > 0) {
      url += '?' + query.join('&');
    }
  }

  return url;
};

TouchApiClient.prototype._request = function(path, method, body, params, unauthorized) {
  var url = this._getUrl(path, params);
  var headers = {};
  var contentType = this._config.contentType;

  if (body) {
    if (this._config.contentType === 'application/x-www-form-urlencoded') {
      var str = [];
      for (var p in body) {
        if (body.hasOwnProperty(p)) {
          str.push(encodeURIComponent(p) + "=" + encodeURIComponent(body[p]));
        }
      }
      body = str.join("&");
    }
    else if (this._config.contentType === 'application/json') {
      body = JSON.stringify(body);
    }
  }

  var self = this;
  return new Promise(function(resolve, reject) {
    function makeRequest() {
      $.ajax(url, {
        type: method,
        data: body,
        contentType: contentType,
        headers: headers
      })
      .done(function(data) {
        resolve(data);
      })
      .fail(function(e) {
        var error = {
          code: e.status,
          message: e.responseText
        };
        reject(error);
      });
    }

    if (unauthorized) {
      return makeRequest();
    }

    self._tokenProvider().then(function(token) {
      if (token) {
        headers.Authorization = 'Bearer ' + token;
      }

      makeRequest();
    }, reject);
  });
};

TouchApiClient.prototype._upload = function(path, file) {
  var url = this._getUrl(path);
  var headers = {};

  var self = this;
  return new Promise(function(resolve, reject) {
    function makeRequest() {
      var data = new FormData();
      data.append('file', file);

      $.ajax(url, {
        type: 'POST',
        data: data,
        cache: false,
        dataType: 'json',
        processData: false,
        contentType: false,
        headers: headers
      })
      .done(function(data) {
        resolve(data);
      })
      .fail(function(jqXHR, textStatus, errorThrown) {
        var error = {
          code: jqXHR.status || 503,
          message: textStatus
        };

        if (jqXHR.responseText) {
          try
          {
            error = JSON.parse(jqXHR.responseText);
          }
          catch(e) {
            error.message = jqXHR.responseText;
          }
        }

        reject(error);
      });
    }

    self._tokenProvider().then(function(token) {
      if (token) {
        headers.Authorization = 'Bearer ' + token;
      }

      makeRequest();
    }, reject);
  });
};

module.exports = TouchApiClient;
